      SUBROUTINE TYPE204(TIME,XIN,OUT,T,DTDT,PAR,INFO,ICNTRL,*)

!**********************************************************************************
!* This program calculates temperatures at 729 grid points (there are 9 nodes in each *
!* direction) of a 3-dimensional wall element,when the user gives the wall dimensions,inside*
!* temperature, and the conductance and capacitance at each node	**
!*
!* Created by Piia Lamberg, at Helsinki University of Technology
!* 
!* Revision history: 
!* 2005-11-08 DAA - Updated to TRNSYS16 standard.
!*										    												
!**********************************************************************************
!
!         
!             TIME = simulation time (not used in this program)
!                T = (not used in this program)
!             DTDT = (not used in this program)
!
!             INFO = information vector in this program
!
!		 INFO(6) = Number of outputs of Type101)
!		 INFO(9) = Type101 is the dynamical model
!
!           ICNTRL =  information vector of the TRNSYS program (not used in this program)
!
!              XIN = Wall component input vector:
!
!           XIN(1) = (room temperature in C)
!		  XIN(2) = Intens [W/m2] (radiation power on the wall in W/m2)
!
!              OUT = Wall component output vector:
!
!		  OUT(1) = Ttulk [C] (average surface temperature of the wall)
!		  OUT(2) = Ceff1(5,5,5) (control variable, not used)
!	   OUT(3-11) = Temperature distribution along the wall normal)
!	  OUT(12-21) = Values of the variable TILA at different points
!
!              PAR = Wall component parameterst:
!
!           PAR(1) = iter (Number of iterations for one time step)    
!           PAR(2) = TtrSul [C] (Melting point of the phase transition compound)
!           PAR(3) = Ttrkit [C] (Crystallization temperature of the compound)
!           PAR(4) = Tdiff [C] (0.5 * the range of melting or crystallization of the compound)
!		  PAR(5) = 0.001*Latentti [J/kg] (0.001*Latentti [J/kg] ( 0.001 * latent heat of melting)
!           PAR(6) = ParTih [kg/m3] (density of the compound)
!           PAR(7) = ParCp [J/kgK] ( Heat capacity of the compound in solid or liquid)
!		  PAR(8) = KavTih [kg/m3] (Density of the matrix compound at a node) 
!		  PAR(9) = KavCp [J/kgK] (Heat capacity of the matrix compound at a node)
!		  PAR(10) = Volume fraction of the phase transition compound at PCM node
!           PAR(11) = Logical unit number for file 'NodeTemp.txt'
!           PAR(12) = Logical unit number for file 'Tile.txt'
!           PAR(13) = Logical unit number for file 'Coeff.txt'
!           PAR(14) = Logical unit number for file 'ALKU.txt'
!
! TRNSYS- TYPE101 variables:
! 
!           LASVALI = time step of calculation [s]
!			   dT = time step of calculation [h]
!			  vrk = number of days in the calculation 
!			tunti = number of hours in the calulation
!			loppu = the last time step of a day
!         MENKERROI = method coefficient of the calculation
!          i_suunta = number of nodes in i direction (width)
!          j_suunta = number of nodes in j direction (height)
!          k_suunta = number of nodes in k direction (depth)
!            LEVEYS = width of the grid [m]
!           KORKEUS = height of the grid [m]
!            SYVYYS = depth of the grid [m]
!      MATKERROS(6) = distance between the nodes [m]               
!              A(6) = areas of the faces of the grid [m2]
!                 V = volume of a node [m3]
!         LAMDA(10) = thermal conductivity of the material [W/mK]
!            Cp(10) = heat capacity of the material [J/kgK]                    
!            Ro(10) = density of the material [kg/m3]
!     TYYPPI(9,9,9) = material type at a node (concrete, steel, PCM...)
!             HILMA = convective heat transfer coefficient of the surface [W/m2K]
!     KAPASI(9,9,9) = heat capacities of nodes  [J/K]
!      G(1000,1000) = conductance between nodes  [W/K]
!     MATA(730,730) = coefficient matrix of the temperature vector for the next time step
!     MATEHO(730,1) = tulevan aika-askeleen tehomatriisi
!     MATB(730,730) = coefficient matrix of the temperature vector for the present time step
!     MBTEHO(730,1) = power matrix for the next time step
!      TULOS(730,1) = temperature vector [C]
!		    ZoneT = temperature of the inside air with sinusoidal variation
!  TULOS1..4(730,1) = temperature vectors [C]
!        Tnyk(81,1) = temperature vector at surface nodes for the present time step [C]
!        Ttul(81,1) = temperature vector at surface nodes for the next time step [C]
!             TnykK = average temperature of surface nodes for the present time step [C]
!             TtulK = average temperature of surface nodes for the next time step [C]
!               Tol = difference of the average temperatures of surface nodes between successive time !steps [C]             
!                 P = heat load on a surface node [W/m2]
!			 Esit = energy per unit area absorbed by the grid from the inside air within one time step [kJ/m2] 
!			 Eluo = energy per unit area emitted by the grid [kJ/m2] 
!			 Etot = difference between absorbed and emitted energies                
!		   TtrSul = melting point of the phase transition compound [C]	
!		   TtrKit = crystallization temperature [C]
!			Tdiff = half of the melting or crystallization range [C]
!		 Latentti = melting heat of the phase transition compound [J/kg]	
!	  TILA(9,9,9) = ratio of the latent heat to the total melting heat 	
!			    n = loop variable
!__________________________________________________________________________________________________
!
! description of the sub-routines: TYPE 101:
!
!			 ALKUAR - Aliohjelma lukee muuttujille alkuarvot tiedostosta.
!
!			 ALKUVARASTO - subroutin calculated the amount of melting of material in the nods.					   sek asettaa faasimuuotsaineen olomuodon alkutilassa kiinteksi.
!
!			 SOLMUOM - Aliohjelma laskee solmupisteen tilavuuden, tahkojen alat sek solmu-
!					   pisteiden vliset etisyydet.
!
!              KONDUKTANSSI - Aliohjelma laskee solmupisteiden vliset konduktanssit.
!
!              KAPASITANSSI - Aliohjelma laskee solmupisteiden lmpkapasiteetit.
!
!		     EFFLAMPOK - Subprogram calculates the effectivc thermal capacity. 
!						 It takes in the acount the supercooling of the "substance changing 
!						 the phase" and the change of the latent heat at the "knot"-point when 
!                          the "substance changing the phase" melts or crystallize.
!		
!			 MATRIISIT - Aliohjelma muodostaa lmptilamatriisien nykyisen ja tulevan 
!					     aika-askeleen kerroinmatriisit.
!
!			 TEHOMAT - Aliohjelma muodostaa hilakopin pintaan osuvan steilykuorman tehomatriisin.
!
!			 MATKER - Subprogram calculates the multiplication of the matrix.
!
!			 MATSUMMA - Aliohjelma laskee matriisien summan.
!
!			 MATERO - Aliohjelma laskee matriisien erotuksen.
!              
!			 PARSE - Aliohjelma knt tulevien lmptilojen kerroinmatriisin.
!
!			 ASUB - Aliohjelman 'SRARSE' aliohjelma, joka laskee matriisin ja vektorin tulon.
!
!              ATSUB - Aliohjelman 'SPARSE' aliohjelma, joka laskee transponoidun matriisin ja 
!					 vektorin tulon.
!
!			 ENERGIA - Aliohjelma laskee hilakopin ja ilman vlisen konvektiivisen lmpvirran
!				       perusteella seinn sitoman ja luovuttaman energian pinta-alayksikk kohti.
!
!_____________________________________________________________________________________________
! Copyright  2005.  All rights reserved.

      !export this subroutine for its use in external DLLs.
      !DEC$ATTRIBUTES DLLEXPORT :: TYPE204

C    TRNSYS acess functions (allow to acess TIME etc.) 
      USE TrnsysConstants
      USE TrnsysFunctions

C-----------------------------------------------------------------------------------------------------------------------
C    TRNSYS DECLARATIONS
      IMPLICIT NONE			!REQUIRES THE USER TO DEFINE ALL VARIABLES BEFORE USING THEM

	DOUBLE PRECISION XIN	!THE ARRAY FROM WHICH THE INPUTS TO THIS TYPE WILL BE RETRIEVED
	DOUBLE PRECISION OUT	!THE ARRAY WHICH WILL BE USED TO STORE THE OUTPUTS FROM THIS TYPE
	DOUBLE PRECISION TIME	!THE CURRENT SIMULATION TIME - YOU MAY USE THIS VARIABLE BUT DO NOT SET IT!
	DOUBLE PRECISION PAR	!THE ARRAY FROM WHICH THE PARAMETERS FOR THIS TYPE WILL BE RETRIEVED
	DOUBLE PRECISION STORED !THE STORAGE ARRAY FOR HOLDING VARIABLES FROM TIMESTEP TO TIMESTEP
	DOUBLE PRECISION T		!AN ARRAY CONTAINING THE RESULTS FROM THE DIFFERENTIAL EQUATION SOLVER
	DOUBLE PRECISION DTDT	!AN ARRAY CONTAINING THE DERIVATIVES TO BE PASSED TO THE DIFF.EQ. SOLVER
	INTEGER*4 INFO(15)		!THE INFO ARRAY STORES AND PASSES VALUABLE INFORMATION TO AND FROM THIS TYPE
	INTEGER*4 NP,NI,NOUT,ND	!VARIABLES FOR THE MAXIMUM NUMBER OF PARAMETERS,INPUTS,OUTPUTS AND DERIVATIVES
	INTEGER*4 NPAR,NIN,NDER	!VARIABLES FOR THE CORRECT NUMBER OF PARAMETERS,INPUTS,OUTPUTS AND DERIVATIVES
	INTEGER*4 IUNIT,ITYPE	!THE UNIT NUMBER AND TYPE NUMBER FOR THIS COMPONENT
	INTEGER*4 ICNTRL		!AN ARRAY FOR HOLDING VALUES OF CONTROL FUNCTIONS WITH THE NEW SOLVER
	INTEGER*4 NSTORED		!THE NUMBER OF VARIABLES THAT WILL BE PASSED INTO AND OUT OF STORAGE
	CHARACTER*3 OCHECK		!AN ARRAY TO BE FILLED WITH THE CORRECT VARIABLE TYPES FOR THE OUTPUTS
	CHARACTER*3 YCHECK		!AN ARRAY TO BE FILLED WITH THE CORRECT VARIABLE TYPES FOR THE INPUTS
C-----------------------------------------------------------------------------------------------------------------------

C-----------------------------------------------------------------------------------------------------------------------
C    USER DECLARATIONS - SET THE MAXIMUM NUMBER OF PARAMETERS (NP), INPUTS (NI),
C    OUTPUTS (NOUT), AND DERIVATIVES (ND) THAT MAY BE SUPPLIED FOR THIS TYPE
      PARAMETER (NP=14,NI=2,NOUT=21,ND=0,NSTORED=0)
C-----------------------------------------------------------------------------------------------------------------------

C-----------------------------------------------------------------------------------------------------------------------
C    REQUIRED TRNSYS DIMENSIONS
      DIMENSION XIN(NI),OUT(NOUT),PAR(NP),YCHECK(NI),OCHECK(NOUT),
	1   STORED(NSTORED),T(ND),DTDT(ND)
      INTEGER NITEMS
	INTEGER LU_NT,LU_T,LU_C,LU_ALKU

	CHARACTER (len=maxMessageLength) FileErr
	LOGICAL FoundFile/.FALSE./
	
      EXTERNAL ALKUAR,KONDUKTANSSI,EFFLAMPOK,KAPASITANSSI,MATRIISIT,
     &         MATKER,SOLMUOM,PARSE,ASUB,ATSUB,ENERGIA,ALKUVARASTO,
     &		 TEHOMAT,MATSUMMA,MATERO		 
      INTEGER, PARAMETER :: tarkkuus = SELECTED_REAL_KIND(7,12)
	INTEGER  n,TYYPPI(9,9,9),i_suunta,j_suunta,k_suunta,i,j,k,lkm,iter
     &		 
	REAL (tarkkuus) :: LAMDA(10),Cp(10),Ro(10),
     &	               KORKEUS,LEVEYS,SYVYYS,MATKERROS(6),A(6),V,
     &                   HILMA,MENKERROI,LASVALI,
     &                   G(1000,1000),KAP1(9,9,9),KAP2(9,9,9),
     &                   Ceff1(9,9,9),Ceff2(9,9,9),      
     &                   MATA(730,730),MATB(730,730),MATEHO(730,1),
     &                   MBTEHO(730,1),
     &                   TULOS(730,1),TULOS1(730,1),TULOS2(730,1),
     &                   TULOS3(730,1),TULOS4(730,1),Tnyk(81,1),
     &                   Ttul(81,1),TnykK,TtulK,Tol,P,vrk,tunti,	
     &                   Pi,dT,ZoneT,Esit,Eluo,Etot,TtrSul,TtrKit,Tdiff,
     &                   Latentti,loppu,Ekok,TILA(9,9,9),ParTih,
     &				   ParCp,KavTih,KavCp,TilOsuus, Intens 				   

	DOUBLE PRECISION TIME0,DELT,TIMEF
      
C    SET THE INPUT AND OUTPUT ARRAYS

      DATA YCHECK/'TE1','IR2'/
      DATA OCHECK/
     1'TE1','DM1','TE1','TE1','TE1','TE1','TE1','TE1','TE1','TE',
     1'TE1','DM1','DM1','DM1','DM1','DM1','DM1','DM1','DM1','DM',
	1'DM1'/

	DATA FoundFile/.FALSE./
	DATA FileErr/'Unable to find file'/
C--------------------------------------------------------------------------------------------------
C--------------------------------------------------------------------------------------------------
C    GET GLOBAL TRNSYS SIMULATION VARIABLES
      TIME0 = getSimulationStartTime()
      DELT  = getSimulationTimeStep()
	TIMEF = getSimulationStopTime()

C--------------------------------------------------------------------------------------------------
C    SET THE VERSION INFORMATION FOR TRNSYS
      IF(INFO(7).EQ.-2) THEN
	  INFO(12)=16
	  RETURN 1
	ENDIF

C-------------------------------------------------------------------------------------------------
C    DO ALL THE VERY LAST CALL OF THE SIMULATION MANIPULATIONS HERE
      IF (INFO(8).EQ.-1) THEN
	   RETURN 1
	ENDIF
C------------------------------------------------------------------------------------------------

C------------------------------------------------------------------------------------------------
C    PERFORM ANY POST-CONVERGENCE MANIPULATIONS THAT ARE REQUIRED HERE
      IF (INFO(13).GT.0) THEN
	   RETURN 1
	ENDIF
C
C-----------------------------------------------------------------------------------------------------------------------

C-----------------------------------------------------------------------------------------------------------------------
C    DO ALL THE VERY FIRST CALL OF THE SIMULATION MANIPULATIONS HERE
      IF (INFO(7).EQ.-1) THEN

C       SET SOME INFO ARRAY VARIABLES TO TELL THE TRNSYS ENGINE HOW THIS TYPE IS TO WORK
         INFO(6)=NOUT				
         INFO(9)=1				
	   INFO(10)=0	!STORAGE FOR VERSION 16 HAS BEEN CHANGED				

C       SET THE REQUIRED NUMBER OF INPUTS, PARAMETERS AND DERIVATIVES THAT THE USER SHOULD SUPPLY IN THE INPUT FILE
C       IN SOME CASES, THE NUMBER OF VARIABLES MAY DEPEND ON THE VALUE OF PARAMETERS TO THIS MODEL....
         NIN=NI
	   NPAR=NP
	   NDER=ND
	       
C       CALL THE TYPE CHECK SUBROUTINE TO COMPARE WHAT THIS COMPONENT REQUIRES TO WHAT IS SUPPLIED IN 
C       THE TRNSYS INPUT FILE
	   CALL TYPECK(1,INFO,NIN,NPAR,NDER)

C       SET THE NUMBER OF STORAGE SPOTS NEEDED FOR THIS COMPONENT
         NITEMS=0
C	   CALL SET_STORAGE_SIZE(NITEMS,INFO)

C       RETURN TO THE CALLING PROGRAM
         RETURN 1

      ENDIF
C-----------------------------------------------------------------------------------------------------------------------


C-----------------------------------------------------------------------------------------------------------------------
C    DO ALL OF THE INITIAL TIMESTEP MANIPULATIONS HERE - THERE ARE NO ITERATIONS AT THE INTIAL TIME
      IF (TIME .LT. (getSimulationStartTime() +
     .   getSimulationTimeStep()/2.D0)) THEN

C       SET THE UNIT NUMBER FOR FUTURE CALLS
         IUNIT=INFO(1)
         ITYPE=INFO(2)

C       CHECK THE PARAMETERS FOR PROBLEMS AND RETURN FROM THE SUBROUTINE IF AN ERROR IS FOUND
C       CHECK THE PARAMETERS FOR PROBLEMS AND RETURN FROM THE SUBROUTINE IF AN ERROR IS FOUND
	   iter     = JFIX(PAR(1)+0.1)	   !Number of iterations
	   TtrSul   = PAR(2)      !TtrSul = melting point of the phase transition of componant[C]
	   TtrKit   = PAR(3)	   !TtrKit = crystallization point of the componant [C]
	   Tdiff    = PAR(4)/2    !Tdiff = 0.5*the range of melting or crystallization of component [C]
	   Latentti = 1000*PAR(5) !Latentti = latent heat of melting of PCM material [J/Kg] 
	   ParTih   = PAR(6)      !Density of the paraffin [kg/m3]
	   ParCp    = PAR(7)	   !Heat capacity of the paraffin in solide or liquide [J/kgK]
	   KavTih   = PAR(8)      !Density of the matrix PCM at a node [kg/m3]
	   KavCp    = PAR(9)      !Heat capacity of the matrix PCM at a node [J/kgK]
	   TilOsuus = PAR(10)     !Volume fraction of the phase transition compound at PCM node (PCM=paraffin)
         LU_NT    = JFIX(PAR(11)+0.1)	  !Logical number for node Temperature file
	   LU_T     = JFIX(PAR(12)+0.1)   !Logical number for TILE values
	   LU_C     = JFIX(PAR(13)+0.1)   !Logical number for COEFF values
	   LU_ALKU  = JFIX(PAR(14)+0.1)   !Logical number for COEFF values

C       PERFORM ANY REQUIRED CALCULATIONS TO SET THE INITIAL VALUES OF THE OUTPUTS HERE
	   OUT(1:NOUT)=0.d0

C       PERFORM ANY REQUIRED CALCULATIONS TO SET THE INITIAL STORAGE VARIABLES HERE
         NITEMS=0
C	   STORED(1)=...

C       PUT THE STORED ARRAY IN THE GLOBAL STORED ARRAY
C         CALL SET_STORAGE_VARS(STORED,NITEMS,INFO)

C     INITIALIZATION OF FILES

C       Initial temperature of the nodes, [C]
        INQUIRE(LU_NT,EXIST=FoundFile)
        IF(.NOT.FoundFile) CALL MESSAGES(-1,FileErr,'fatal',IUNIT,ITYPE)   
        DO i=1,729
          WRITE (LU_NT,'(F14.10)')20.0000000000
        END DO
	!
	!State of PCM at the beginning of the simulation, [0...1]
	!State = 0: Completely solid PCM
	!0 < State < 1 : Partly melted PCM
	!State = 1: Completely melted PCM
	!
        INQUIRE(LU_T,EXIST=FoundFile)
        IF(.NOT.FoundFile) CALL MESSAGES(-1,FileErr,'fatal',IUNIT,ITYPE)   
       
        DO i=1,729
          WRITE (LU_T,'(F14.10)')0.0000000000
        END DO
	!
	!Effective heat capacity of PCM, [J/kgK]
        INQUIRE(LU_C,EXIST=FoundFile)
        IF(.NOT.FoundFile) CALL MESSAGES(-1,FileErr,'fatal',IUNIT,ITYPE)   
        DO i=1,729
          WRITE (LU_C,'(F14.5)')2200.00000
        END DO


C       RETURN TO THE CALLING PROGRAM
         RETURN 1

      ENDIF
C-----------------------------------------------------------------------------------------------------------------------

C-----------------------------------------------------------------------------------------------------------------------
C    *** ITS AN ITERATIVE CALL TO THIS COMPONENT ***
C-----------------------------------------------------------------------------------------------------------------------

C    RE-READ THE PARAMETERS IF ANOTHER UNIT OF THIS TYPE HAS BEEN CALLED SINCE THE LAST 
C    TIME THEY WERE READ IN
      IF(INFO(1).NE.IUNIT) THEN
        !recall the UNIT and TYPE number
        IUNIT = INFO(1)
	  ITYPE = INFO(2)
        !read parameter values
	   iter     = PAR(1)	   !Number of iterations
	   TtrSul   = PAR(2)      !TtrSul = melting point of the phase transition of componant[C]
	   TtrKit   = PAR(3)	   !TtrKit = crystallization point of the componant [C]
	   Tdiff    = PAR(4)/2    !Tdiff = 0.5*the range of melting or crystallization of component [C]
	   Latentti = 1000*PAR(5) !Latentti = latent heat of melting of PCM material [J/Kg] 
	   ParTih   = PAR(6)      !Density of the paraffin [kg/m3]
	   ParCp    = PAR(7)	   !Heat capacity of the paraffin in solide or liquide [J/kgK]
	   KavTih   = PAR(8)      !Density of the matrix PCM at a node [kg/m3]
	   KavCp    = PAR(9)      !Heat capacity of the matrix PCM at a node [J/kgK]
	   TilOsuus = PAR(10)     !Volume fraction of the phase transition compound at PCM node (PCM=paraffin)
         LU_NT    = JFIX(PAR(11)+0.1)	  !Logical number for node Temperature file
	   LU_T     = JFIX(PAR(12)+0.1)   !Logical number for TILE values
	   LU_C     = JFIX(PAR(13)+0.1)   !Logical number for COEFF values
	   LU_ALKU  = JFIX(PAR(14)+0.1)   !Logical number for COEFF values
	ENDIF

C--------------------------------------------------------------------------------------------------
C    GET AND CHECK INPUT VALUES

      ZoneT  = XIN(1)	!Room temperature
      Intens = XIN(2) !Radiation power on the wall [W/m2]


499   CALL ALKUAR(TULOS,KORKEUS,LEVEYS,SYVYYS,i_suunta,j_suunta,
     &            k_suunta,TYYPPI,LAMDA,Cp,Ro,HILMA,LASVALI,
     &            MENKERROI,Ceff1,Ceff2,TILA,LU_NT,LU_T,LU_C,LU_ALKU)  
	!
	!
	TULOS(730,1)= ZoneT
      !

      !Calculation of the grid's dimension 
	CALL SOLMUOM(KORKEUS,LEVEYS,SYVYYS,i_suunta,j_suunta,k_suunta,
     &             V,A,MATKERROS)
      !
      !Calculation of the conductance between the nods
	CALL KONDUKTANSSI(LAMDA,A,MATKERROS,G,HILMA,TYYPPI)
      !
	!Mritelln PCM-aineen alkutila (kiinte) sek energiamr joka voi varastoitua 
	!solmupisteeseen sulamisalueessa [Ttr-Tdiff,Ttr+Tdiff], jossa on PCM:. 
	!(the type of material 9 = PCM)
	CALL ALKUVARASTO(Cp,Ro,V,Tdiff,LATENTTI,Ekok,ParTih,TilOsuus)
	!
      !Number of iteration of program   DO loopissa
      lkm=0
	!
      !Caculation the number of repeated  DO loopin lopussa olevan IF lauseen mrm lkm.
	DO 
      lkm=lkm+1
	        !Calculation the repeated of the heat capacity of the nods 
		    CALL KAPASITANSSI(Cp,Ro,V,Ceff1,Ceff2,TYYPPI,KAP1,KAP2,
     &					      ParTih,KavCp,KavTih,TilOsuus)    
		    !
	        !Calculate the coefficient matrix  MATA and MATB  
		    CALL MATRIISIT(MATA,MATB,MENKERROI,LASVALI,KAP1,KAP2,G) 
	        !
	        !Calculation of  kertolasku: MATB*T=TULOS1	
	        CALL MATKER(MATB,TULOS,TULOS1)
	        !
              !Heat load  of adsorption of one grid [W]. the number of repeating the puissance :n iteraatio- 
              !kierroksilla. 
              IF ((lkm.LE.iter).AND.(INFO(7).LE.0)) THEN
              P=Intens*A(1)
              ELSE
              P=0
			END IF
	        ! MAteho and MBteho
	        CALL TEHOMAT(P,LASVALI,MENKERROI,KAP1,KAP2,MATEHO,MBTEHO)
	        !
	        !Calculation  the somme: MBteho+TULOS1=TULOS2
	        CALL MATSUMMA(MBTEHO,TULOS1,TULOS2)
	        !
	        !Calculation of the difference (cart) (gap): TULOS2-MAteho=TULOS3
	        CALL MATERO(TULOS2,MATEHO,TULOS3)		
	        !
	        ! the process of the system  MATA*TULOS4=TULOS3 TULOS3 Temperature vector  TULOS4 situated 
	        !!! Time- step vector TULOS4 alkuarvoiksi 0 
	        !(Subroutin 'Sparse' toiminta perustuu iteraatioihin)
	          DO n=1,730
	              TULOS4(n,1)=0
	          END DO
	        CALL PARSE(TULOS3,730,TULOS4,MATA)
	       !
	       !Talletetaan heat surfacique vector  'Tnyk' and 'Ttul'
	        DO n=1,81
	           Tnyk(n,1)=TULOS(n,1)
	           Ttul(n,1)=TULOS4(n,1)
	        END DO
		   !
	       !'TnykK'
	        TnykK=sum(Tnyk)/81
	       !
	       !Calculation of the average of present temperature at the surface !!  'TtulK'
	        TtulK=sum(Ttul)/81
	       !
	       !Calculation of the heat capacity for the next time step 
		   !and approximation of the (position) statement of the PCM	
	        CALL Efflampok(TULOS4,TULOS,Ceff1,Ceff2,TILA,TtrSul,TtrKit,
     &					   Tdiff,Latentti,Ro,V,Ekok,ParCp,ParTih,
     &					   TilOsuus)	
	       !
		   !The new value of the initial temperature vector !!
	       !'Tvanha'
		    DO n=1,730
       	      TULOS(n,1)=TULOS4(n,1)
		    END DO
	       !
		   !The Puissance totale of the wall element [kJ]','Eluo [kJ]' and 'Etot = Esit + Eluo [kJ]' energia
			CALL ENERGIA(VRK,TUNTI,LASVALI,HILMA,TnykK,Zonet,
     &			         Esit,Eluo,Etot)
		   !
		   !
        IF (lkm.GE.iter) EXIT
	END DO
	!
      ! Program ulostulo TRNSYS:iin
      !
      !Control of the values of the surface temperature of the wall elements !! TRNSYS:iin
      OUT(1) = TtulK
	!(5,5,5)
	OUT(2) = Ceff1(5,5,5)
	!Nod temperature (1,1,1) 
	OUT(3) = TULOS(1,1)
	!Nod temperature (1,1,2) 
	OUT(4) = TULOS(82,1)
	!Nod temperature (1,1,3) 
	OUT(5) = TULOS(163,1)
	!Nod temperature (1,1,4) 
	OUT(6) = TULOS(244,1)
	!Nod temperature (1,1,5) 
	OUT(7) = TULOS(325,1)
	!Nod temperature (1,1,6) 
	OUT(8) = TULOS(406,1)
	!Nod temperature (1,1,7)
	OUT(9) = TULOS(487,1)
	!Nod temperature (1,1,8) 
	OUT(10) = TULOS(568,1)
	!Nod temperature (1,1,9) 
	OUT(11) = TULOS(649,1)
	!state of 9 PCM layers 
	OUT(12) = TILA(1,1,1)
	OUT(13) = TILA(1,1,2)
	OUT(14) = TILA(1,1,3)
	OUT(15) = TILA(1,1,4)	 
	OUT(16) = TILA(1,1,5)	 
	OUT(17) = TILA(1,1,6)	 
	OUT(18) = TILA(1,1,7)	 
	OUT(19) = TILA(1,1,8)	 
	OUT(20) = TILA(1,1,9)	 
      !
      ! To check the valus of nods  
      ! 
      !(TRNSYS:n)
	REWIND(LU_NT)
      DO n=1,729
	  WRITE(LU_NT,'(F14.10)')TULOS4(n,1)
      END DO
	!
	!
	REWIND(LU_T)!
      do k=1,9
        	do j=1,9
        		do i=1,9
				WRITE(LU_T,'(F14.10)')TILA(i,j,k)
        		end do
        	end do
      end do	

	! the heat capacity of nods	
	REWIND(LU_C)
      do k=1,9
        	do j=1,9
        		do i=1,9
				WRITE(LU_C,'(F14.5)')Ceff1(i,j,k)
        		end do
        	end do
      end do	
      RETURN 1
	END


!_________________________________________________________________________________________
	!
	!Aliohjelma, joka avaa alkuarvotiedoston ja lukee alkuarvot ohjelman kyttn
	!
      !Alkuarvotiedosto (9,9,9) hilan tapauksessa
      !
	SUBROUTINE ALKUAR(T,KORKEUS,LEVEYS,SYVYYS,i_suunta,j_suunta,
     &           k_suunta,TYYPPI,LAMDA,Cp,Ro,HILMA,LASVALI,MENKERROI,
     &           Ceff1,Ceff2,TILA,LU_NT,LU_T,LU_C,LU_ALKU)

      USE TrnsysFunctions

      IMPLICIT NONE 
      INTEGER, PARAMETER :: tarkkuus = SELECTED_REAL_KIND(7,12)
	INTEGER, INTENT(OUT) :: i_suunta,j_suunta,k_suunta,TYYPPI(9,9,9)
	INTEGER LU_NT,LU_T,LU_C,LU_ALKU

	REAL (tarkkuus), INTENT(OUT) :: T(730,1),KORKEUS,LEVEYS,SYVYYS,
     & 	                            LAMDA(10),Cp(10),Ro(10),LASVALI,
     &							    MENKERROI,HILMA,Ceff1(9,9,9),
     &                                Ceff2(9,9,9),TILA(9,9,9)
      CHARACTER :: apu
	INTEGER :: i,j,k
      REAL (tarkkuus) :: Tapu,Ceff,nodetemp        
      !
	!

c      LU_A = getNextAvailableLogicalUnit()

c      OPEN (LU_A, FILE = 'alku.txt', STATUS ='OLD')
	REWIND(LU_ALKU)

      !
      ! Knowing the characters of every material type  
      DO k=1,9
201     READ (LU_ALKU,'(A)')apu
        IF (apu.EQ.'C') THEN
          GOTO 201
        ELSE
          BACKSPACE (LU_ALKU)  
          DO j=1,9
             READ(LU_ALKU,'(4X,9I3)')
     &           TYYPPI(1,j,k),TYYPPI(2,j,k),TYYPPI(3,j,k),
     &           TYYPPI(4,j,k),TYYPPI(5,j,k),TYYPPI(6,j,k),
     &           TYYPPI(7,j,k),TYYPPI(8,j,k),TYYPPI(9,j,k)               
		END DO
        END IF
	END DO
      !
	! The three values specific (3 kpl) for the 10 material type  (10 kpl) 
202   READ (LU_ALKU,'(A)')apu
      IF (apu.EQ.'C') THEN
        GOTO 202
      ELSE
        BACKSPACE (LU_ALKU)  
        DO i=1,10
          READ (LU_ALKU,'(4X,F5.3,F5.1,F7.2)')LAMDA(i),Cp(i),Ro(i) 
        END DO
      END IF
      !
	! Knowing the dimension of every layer at the nods
203   READ (LU_ALKU,'(A)')apu
      IF (apu.EQ.'C') THEN
        GOTO 203
      ELSE
        BACKSPACE (LU_ALKU)  
	  READ(LU_ALKU,'(4X,3I2)')i_suunta,j_suunta,k_suunta
      END IF
      !
	! Knowing the dimension of the wall
204   READ (LU_ALKU,'(A)')apu
      IF (apu.EQ.'C') THEN
        GOTO 204
      ELSE
        BACKSPACE (LU_ALKU)  
	  READ(LU_ALKU,'(4X,3F6.0)')KORKEUS,LEVEYS,SYVYYS
      END IF
      !
	! Knowing the convective heat transfer coefficient of the surface 
205   READ (LU_ALKU,'(A)')apu
      IF (apu.EQ.'C') THEN
        GOTO 205
      ELSE
        BACKSPACE (LU_ALKU)  
	  READ (LU_ALKU,'(4X,F5.1)')HILMA
      END IF
      !
      ! calculation the time step 
206   READ (LU_ALKU,'(A)')apu
      IF (apu.EQ.'C') THEN
        GOTO 206
      ELSE
        BACKSPACE (LU_ALKU)  
	  READ (LU_ALKU,'(4X,F4.0)')LASVALI
      END IF
      !
      !  
207   READ (LU_ALKU,'(A)')apu
      IF (apu.EQ.'C') THEN
        GOTO 207
      ELSE
        BACKSPACE (LU_ALKU)  
	  READ(LU_ALKU,'(4X,F4.2)')MENKERROI
      END IF
	!
	! knowing all the heat Of nods, and substitution the temperature matrix.
      ! ( all the points at the nods have the same temperature !!).
208   READ (LU_ALKU,'(A)')apu
      IF (apu.EQ.'C') THEN
        GOTO 208
      ELSE
        BACKSPACE (LU_ALKU)  
        READ (LU_ALKU,'(4X,F5.2)')Tapu
	  DO i=1,729
	    T(i,1)=Tapu
	  END DO
      END IF
      !
      ! Ilmapisteen lmptilan lukeminen
209   READ (LU_ALKU,'(A)')apu
      IF (apu.EQ.'C') THEN
        GOTO 209
      ELSE
        BACKSPACE (LU_ALKU)  
        READ (LU_ALKU,'(4X,F5.2)')T(730,1)
      END IF
      !
      ! to check the temperature of the nods 
	  REWIND(LU_NT)
        DO i=1,729
         READ (LU_NT,'(F14.10)')nodetemp
	    T(i,1)=nodeTemp
	  END DO

	
	!
	! To check the variables of 'Tila' at the nods
	  REWIND(LU_T)
        do k=1,9
        	do j=1,9
        		do i=1,9
				READ(LU_T,'(F14.10)') TILA(i,j,k)
        		end do
        	end do
       end do	
	!
	! To check the values of heat capacity in the file
      !  
	  REWIND(LU_C)
        do k=1,9
        	do j=1,9
        		do i=1,9
				READ(LU_C,'(F14.5)')Ceff1(i,j,k)
				Ceff2(i,j,k) = Ceff1(i,j,k)
        		end do
        	end do
        end do	!
      ! 
C	CLOSE (LU_ALKU,STATUS='keep')
 	RETURN
	END SUBROUTINE
	!_________________________________________________________________________
      !
	!	Program, for the step of the volume fraction V at the nod, 
      !   !! A and the material coefficient !!  MATKERROS.
	!
	SUBROUTINE SOLMUOM(KORKEUS,LEVEYS,SYVYYS,i_suunta,j_suunta,
     &                   k_suunta,V,A,MATKERROS)
	IMPLICIT NONE
      INTEGER, PARAMETER :: tarkkuus = SELECTED_REAL_KIND(7,12)
	INTEGER, INTENT(IN) :: i_suunta,j_suunta,k_suunta
	REAL (tarkkuus), INTENT(IN) :: KORKEUS,LEVEYS,SYVYYS  
	REAL (tarkkuus), INTENT(OUT) :: V,A(6),MATKERROS(6)
      !
	V=(KORKEUS/j_suunta)*(LEVEYS/i_suunta)*(SYVYYS/k_suunta)
	!
	!The depth direction (k_ direction)
	A(1)=(KORKEUS/j_suunta)*(LEVEYS/i_suunta)
	A(2)=(KORKEUS/j_suunta)*(LEVEYS/i_suunta)
      !
	!The height direction (j_ direction)
	A(3)=(LEVEYS/i_suunta)*(SYVYYS/k_suunta)
	A(4)=(LEVEYS/i_suunta)*(SYVYYS/k_suunta)
      !
	!The width direction ( (i_ direction)
	A(5)=(KORKEUS/j_suunta)*(SYVYYS/k_suunta)
	A(6)=(KORKEUS/j_suunta)*(SYVYYS/k_suunta)
	!
	!The thickness of material layer in the direction of depth (k - direction)
	MATKERROS(1)=SYVYYS/k_suunta
	MATKERROS(2)=SYVYYS/k_suunta
      !
	!The thickness of material layer in the direction of height ( j_ direction)
	MATKERROS(3)=KORKEUS/j_suunta
	MATKERROS(4)=KORKEUS/j_suunta
      !
	! the thickness of material layer in the direction of width (i- direction)
	MATKERROS(5)=LEVEYS/i_suunta
	MATKERROS(6)=LEVEYS/i_suunta
	!
      RETURN
	END SUBROUTINE
	!____________________________________________________________________________
	!
	! Program for calculate the heat conductivity between the nods (2025kpl)
	!
  	SUBROUTINE KONDUKTANSSI(LAMDA,A,MATKERROS,G,HILMA,TYYPPI)
	IMPLICIT NONE
      INTEGER, PARAMETER :: tarkkuus = SELECTED_REAL_KIND(7,12)
	INTEGER, INTENT(INOUT) :: TYYPPI(9,9,9)
	REAL (tarkkuus), INTENT(IN) :: LAMDA(10),A(6),MATKERROS(6),HILMA
	REAL (tarkkuus), INTENT(OUT) :: G(1000,1000)
	INTEGER :: i,j,k,l,m,n,t,u,v,w,apux,apuy(6),apu1,apu2,p
      !	
	! putting the values of G matrix equal zero 	
	DO  i=1,1000
		DO  j=1,1000
			G(i,j)=0
		END DO
	END DO	
	!
	! 
     	! The coordinates of inside points are defined 110		
	!  
	!Apux is calculated from the coordinates of 
	! the knot and apuy from the neighbour coordinates.
      !
	DO i=1,9
		DO j=1,9
			apux=i*100+j*10+1
			apuy(1)=i*100+j*10+2
			apuy(2)=110
			apuy(3)=i*100+(j+1)*10+1
			apuy(4)=i*100+(j-1)*10+1
			apuy(5)=(i+1)*100+j*10+1
			apuy(6)=(i-1)*100+j*10+1
      !
      ! Filtering the values greater than 1000. (We give them value of 100 which are
	! removed with other wrong values of conductance at the end of the program)
      !
              DO p=1,6
                  IF (apuy(p).GT.1000) THEN
                      apuy(p)=100
                  END IF
              END DO 
      !
		    DO t=1,6
				IF (t.EQ.2) THEN
					G(apux,110)=HILMA*A(2)
					G(110,apux)=G(apux,110)
				ELSE
      !
	! Coordinates u,v,w are declared to determine the " type "
	!  and they are calculated by dividing apuy(t) in the way described below.
	!  (without rounding : 11/3=3).
	!
				    u=apuy(t)/100
				    v=(apuy(t)-u*100)/10
				    w=(apuy(t)-u*100-v*10)
	!
	! Furthermore, it have to take acount that only the allowed coordinates
	! has they type defined in the file of initial values.
	! For all wrong coordinates the type of the point 111 is given. By doing this doesn't affect to calculation
	! because the conductances of the wrong coordinates are set to zero at the end of the loop.
      !       
                      IF ((u.LT.1).OR.(u.GT.9).OR.(v.LT.1).OR.(v.GT.9)
     &                    .OR.(w.LT.1).OR.(w.GT.9)) THEN
                          u=1
                          v=1
                          w=1
                      END IF
      ! Node 'apu1'
			    apu1=tyyppi(i,j,1)
      ! Neighbour node 'apu2'
		    	apu2=tyyppi(u,v,w)
	! The conductance between the nodes and the neighbour nodes 	
		    	G(apux,apuy(t))=(LAMDA(apu1)+LAMDA(apu2))/(2*
     &					    	MATKERROS(t))*A(t)
			    G(apuy(t),apux)=G(apux,apuy(t))
			    END IF
			END DO !END DO t=1,6			
		END DO !END DO j=1,9	
	END DO !END DO i=1,9
    	! END Ilmasolmuun yhteydess olevat solmut

	! DEFINING THE KNOTS, WHO ARE NOT AT THE EDGE, TO THE MATRIX 
	! G (=CENTRAL KNOTS) "G (=KESKISOLMUT)"
	!  
	! 
	! 
      !
	DO i=1,9		
		DO j=1,9
			DO k=2,8			
					apux=i*100+j*10+k
					apuy(1)=i*100+j*10+(k-1)
					apuy(2)=i*100+j*10+(k+1)
					apuy(3)=i*100+(j-1)*10+k
					apuy(4)=i*100+(j+1)*10+k
					apuy(5)=(i+1)*100+j*10+k
					apuy(6)=(i-1)*100+j*10+k
      !
      ! Filtering the values greater than 1000. 
	! 
                      DO p=1,6
                        IF (apuy(p).GT.1000) THEN
                          apuy(p)=100
                        END IF
                      END DO 
      !				
	! Coordinates u,v,w are declared to determine the " type " 
	! 
	! 
	! integer (without rounding : 11/3=3)
      !
                     	DO t=1,6       
				       u=apuy(t)/100
					   v=(apuy(t)-u*100)/10
					   w=(apuy(t)-u*100-v*10)
      !
      ! If any of u,v and w is below 1 or greater than 9 they are given the type of the 
	! point 111. The conductances of the wrong coordinates are set to zero at the end
	! of the loop.  
      !
                        IF ((u.LT.1).OR.(u.GT.9).OR.(v.LT.1).OR.(v.GT.9)
     &                     .OR.(w.LT.1).OR.(w.GT.9)) THEN
                           u=1
                           v=1
                           w=1
                        END IF
      ! Node 'apu1'
						apu1=tyyppi(i,j,k)
      ! Neighbour node 'apu2'
						apu2=tyyppi(u,v,w)
      ! Lasketaan solmupisteen ja naapurien vliset konduktanssit.
						G(apux,apuy(t))=(LAMDA(apu1)+LAMDA(apu2))/
     &									(2*MATKERROS(t))*A(t)
						G(apuy(t),apux)=G(apux,apuy(t))
				END DO  !End DO t=1,6
			END DO	!End DO k=2,8
		END DO   !End DO j=1,9
	END DO	!End DO i=1,9
      !
	! "TO DEFINE VALUES OF THE KNOTS IN THE MATRIX G THAT HAVE CONTACT TO " 
	!  THE OUTSIDE  POINT ". 
	! " OUTSIDE POINT " = THE MOST INNER POINTS IN THE WALL (k=9)
      !
	! The outside point is defined point (1000)
	!
	! Additional variables are defined to store the conductances at the middle of 
	! the knots to 2-dimensional table. Apux is calculated from the coordinates of 
	! the knot and apuy from the neighbour coordinates.
      !	
	DO i=1,9
		DO j=1,9
			apux=i*100+j*10+9
			apuy(1)=i*100+j*10+8    
			apuy(2)=1000
			apuy(3)=i*100+(j+1)*10+9
			apuy(4)=i*100+(j-1)*10+9
			apuy(5)=(i+1)*100+j*10+9
			apuy(6)=(i-1)*100+j*10+9
      !
      ! Filtering the values greater than 1000. (We give them value of 100 
	! which are removed with other wrong values of conductance at the end of the program).
      !
              DO p=1,6
                  IF (apuy(p).GT.1000) THEN
                      apuy(p)=100
                  END IF
              END DO 
      !	
			DO t=1,6
				IF (t.EQ.2) THEN
	! Koska nyt ei ole kytss ulkoilmapistett mritetn se
	! nollaksi
					G(apux,1000)=0
					G(1000,apux)=0
				ELSE
      !
	!Coordinates u,v,w are declared to determine the " type " and they are calculated 
	!by dividing apuy(t) in the way described below. 
	! u,v and w are automatically integer (without rounding : 11/3=3).
      !
					u=apuy(t)/100
					v=(apuy(t)-u*100)/10
      				w=(apuy(t)-u*100-v*10)
      ! 
      !
	! 
	! The conductances of the wrong coordinates are set to zero at
	! the end of the loop.
      !
                       IF ((u.LT.1).OR.(u.GT.9).OR.(v.LT.1).OR.(v.GT.9)
     &                    .OR.(w.LT.1).OR.(w.GT.9)) THEN
                         u=1
                         v=1
                         w=1
                       END IF
      ! Node 'apu1'
					apu1=tyyppi(i,j,9)
      ! Neighbour node 'apu2'
					apu2=tyyppi(u,v,w)
	! calculat the echange of conductivity between the node and the neighbour nod
					G(apux,apuy(t))=(LAMDA(apu1)+LAMDA(apu2))/(2*
     &								MATKERROS(t))*A(t)
					G(apuy(t),apux)=G(apux,apuy(t))
	!				
				END IF
			END DO !END DO t=1,6			
		END DO !END DO j=1,9		
	END DO !END DO i=1,9
	! END "ULKOILMAPISTEET"
      !
      ! PUTTING THE VALUE OF THE CONDUCTIVITY ZERO
	do i=1,1000
		do j=1,1000
	        if ((i.LT.110).OR.(j.LT.110)
     &       .OR.(i.GE.200).AND.(i.LE.210).OR.(j.GE.200).AND.(j.LE.210)
     &       .OR.(i.GE.300).AND.(i.LE.310).OR.(j.GE.300).AND.(j.LE.310)
     &       .OR.(i.GE.400).AND.(i.LE.410).OR.(j.GE.400).AND.(j.LE.410)
     &       .OR.(i.GE.500).AND.(i.LE.510).OR.(j.GE.500).AND.(j.LE.510)
     &       .OR.(i.GE.600).AND.(i.LE.610).OR.(j.GE.600).AND.(j.LE.610)
     &       .OR.(i.GE.700).AND.(i.LE.710).OR.(j.GE.700).AND.(j.LE.710)
     &       .OR.(i.GE.800).AND.(i.LE.810).OR.(j.GE.800).AND.(j.LE.810)
     &       .OR.(i.GE.900).AND.(i.LE.910).OR.(j.GE.900).AND.(j.LE.910))
     &        then
			G(i,j)=0
			end if
		end do  !End DO j=1,1000
      end do  !End DO i=1,1000 
      RETURN
	END SUBROUTINE
	!
	!END KONDUKTANSSI
	!__________________________________________________________________________
      !Subprogram 'Efflampok' calculates the effective thermal capacity
      !'Ceff1' and 'Ceff2' 
      !
      !
      !  'Ceff1' on eff.lmpk. next time stepa.
      !  'Ceff2' on eff.lmpk. present time step.
      !
      !
	!
	SUBROUTINE Efflampok(T1,T2,Ceff1,Ceff2,TILA,TtrSul,TtrKit,
     &					 Tdiff,Latentti,Ro,V,Ekok,ParCp,ParTih,
     &					 TilOsuus)	
	IMPLICIT NONE
      INTEGER, PARAMETER :: tarkkuus = SELECTED_REAL_KIND(7,12)
	REAL (tarkkuus), INTENT(IN) :: T1(730,1),T2(730,1),TtrSul,TtrKit,
     &					           Tdiff,Latentti,Ro(10),V,Ekok,
     &							   ParCp,ParTih,TilOsuus							   
	REAL (tarkkuus), INTENT(OUT) :: Ceff1(9,9,9),Ceff2(9,9,9)
	REAL (tarkkuus), INTENT(IN OUT) :: TILA(9,9,9)
	INTEGER i,j,k,n
	REAL (tarkkuus) ::  VanhaT(9,9,9),NykT(9,9,9),UusiT(9,9,9),
     &					Tapu1(9,9,9),Tapu2(9,9,9),dT(9,9,9),
     &				    DiffTILA(9,9,9)
	!
	!calculate the heat matrix 'T1(730,1)' and 'T2(730,1)' kolmiulotteiset
	!taulukot 'NykT(9,9,9)' and 'VanhaT(9,9,9)
      n=0
      DO k=1,9
	    DO j=1,9
		    DO i=1,9
                 n=n+1
      !Taulukko 'NykT' saa matriisin 'T1' arvot matriisin
      !729. alkioon asti.
                 NykT(i,j,k)=T1(n,1)
      !Taulukko 'VanhaT' saa matriisin 'T2' arvot matriisin
      !          729. alkioon asti.
                 VanhaT(i,j,k)=T2(n,1)
      		END DO !END DO i=1,9
		END DO !END DO=j=1,9
	END DO !END DO k=1,9
	!
	DO k=1,9
	   DO j=1,9
		  DO i=1,9
	!The values of State variable    :  TILA=0, IF  PCM State immobilier
	!								    TILA=1, IF PCM melting
	!								    0 < TILA <1, IF PCM transition state
	!
			   IF (TILA(i,j,k).GT.1) THEN
			   TILA(i,j,k) = 1
			   END IF
	!
			   IF (TILA(i,j,k).LT.0) THEN
			   TILA(i,j,k) = 0
			   END IF
      !
      !Etat immobilier ou melting state of PCM    
			   IF ((NykT(i,j,k).LT.(TtrKit-Tdiff))
     &			   .OR.(NykT(i,j,k).GT.(TtrSul+Tdiff))
     &			   .OR.((NykT(i,j,k).GT.(TtrKit+Tdiff))
     &			   .AND.(NykT(i,j,k).LT.(TtrSul-Tdiff))))
     &		   THEN
			   Ceff1(i,j,k) = ParCp
			   Ceff2(i,j,k) = ParCp
			   END IF
      !
	!Energy PCM and Ttr(melting)-dT < T < Ttr(melting)+dT
			   IF ((TILA(i,j,k).LT.1).AND.(NykT(i,j,k).GE.
     &			  (TtrSul-Tdiff)).AND.(NykT(i,j,k).LE.(TtrSul+Tdiff)))
     &		   THEN
			   Ceff1(i,j,k) = (Latentti/(2*Tdiff)) + ParCp
			   Ceff2(i,j,k) = (Latentti/(2*Tdiff)) + ParCp
			   END IF
      !
	!Melting PCM, taux of heat Ttr(melting)-dT < T < Ttr(melting)+dT
			   IF ((TILA(i,j,k).EQ.1).AND.(NykT(i,j,k).GE.
     &			  (TtrSul-Tdiff)).AND.(NykT(i,j,k).LE.(TtrSul+Tdiff)))
     &		   THEN
			   Ceff1(i,j,k) = ParCp
			   Ceff2(i,j,k) = ParCp
			   END IF
	!
	!solidification PCM and Ttr(solidification)-dT < T < Ttr(solidification)+dT
			   IF ((TILA(i,j,k).GT.0).AND.(NykT(i,j,k).GE.
     &			   (TtrKit-Tdiff)).AND.(NykT(i,j,k).LE.
     &		       (TtrKit+Tdiff))) THEN
			   Ceff1(i,j,k) = (Latentti/(2*Tdiff)) + ParCp
			   Ceff2(i,j,k) = (Latentti/(2*Tdiff)) + ParCp
			   END IF
	!
	!solidification PCM, taux Ttr(solidification)-dT < T < Ttr(solidification)+dT
			   IF ((TILA(i,j,k).EQ.0).AND.(NykT(i,j,k).GE.
     &			   (TtrKit-Tdiff)).AND.(NykT(i,j,k).LE.
     &		       (TtrKit+Tdiff))) THEN
			   Ceff1(i,j,k) = ParCp
			   Ceff2(i,j,k) = ParCp
			   END IF		
		 END DO !END DO i=1,9
	   END DO !END DO=j=1,9
	END DO !END DO k=1,9
	!
	!Approximation of the phase inversion; melting;crystalization of PCM material 
	!
	DO i=1,9
		DO j=1,9
			DO k=1,9
			   !
			   !	  
			   IF ((TILA(i,j,k).LT.1)
     &			   .AND.(VanhaT(i,j,k).GE.(TtrSul-Tdiff))
     &			   .AND.(NykT(i,j,k).LE.(TtrSul+Tdiff)))
     &		   THEN
			   !control the PCM material at the melted nods which are connected  
			   !
			   !
			   DiffTILA(i,j,k) = ABS(Latentti*ParTih*TilOsuus*V*
     &			           (NykT(i,j,k)-VanhaT(i,j,k))/(Ekok*2*Tdiff))
			   !			   
			   TILA(i,j,k) = TILA(i,j,k) + DiffTILA(i,j,k)
			   END IF
			   !
			   !
			   !		  			   
			   IF ((TILA(i,j,k).GT.0)
     &			   .AND.(NykT(i,j,k).GE.(TtrKit-Tdiff))
     &			   .AND.(VanhaT(i,j,k).LE.(TtrKit+Tdiff)))
     &		   THEN
			   !
			   !
			   !
			   DiffTILA(i,j,k) = - ABS(Latentti*ParTih*TilOsuus*V*
     &			          (NykT(i,j,k)-VanhaT(i,j,k))/(Ekok*2*Tdiff))
			   !Lasketaan sulamisen vaikutus solmupisteen PCM:n olomuotoon	
			   TILA(i,j,k) = TILA(i,j,k) + DiffTILA(i,j,k)
			   END IF
			END DO !DO k=1,9
		END DO !DO j=1,9
	END DO !DO i=1,9
	RETURN
	END SUBROUTINE
      ! END Efflampok
	!_Subprogram 'Kapasitanssi' laskee lmpkapasiteettien 'KAP1' and 'KAP2'
      !  (729 kpl) value.
      !
	!   'KAP1' is the capacity at the next time step.
      !   'KAP2' is the capacity at the present time step. 
	!  
	! 
	!  
	! 
      !
      ! Variable apu1 has the number of the type at the knot-point. In the if-part 
	! of the subroutine is verified if the knot-point is " material changing the phase 
	! " (type 9). If the knot-point is " material changing the phase ", the capacitance 
	! of the knot is calculated using the effective thermal capacities Coeff1,2.
      !
      ! If the type of the knot-point is different from 9, the capacitance is calculated 
	! using the thermal capicity Cap(apu1) of material normally given in the initial file.
      !
	SUBROUTINE KAPASITANSSI(Cp,Ro,V,Ceff1,Ceff2,TYYPPI,KAP1,KAP2,
     &						ParTih,KavCp,KavTih,TilOsuus)
	IMPLICIT NONE
      INTEGER, PARAMETER :: tarkkuus = SELECTED_REAL_KIND(7,12)
      INTEGER, INTENT(IN) :: TYYPPI(9,9,9)
	REAL (tarkkuus), INTENT(IN) :: Cp(10),Ro(10),V,Ceff1(9,9,9),
     &                               Ceff2(9,9,9),ParTih,KavCp,KavTih,
     &							   TilOsuus
	REAL (tarkkuus), INTENT(OUT) :: KAP1(9,9,9),KAP2(9,9,9) 
	INTEGER i,j,k,apu1
      !
	DO k=1,9
		DO j=1,9
			DO i=1,9
      ! 'apu1' 
      ! [1,10]
			            apu1=tyyppi(i,j,k)
                        IF (apu1.EQ.9) THEN
      ! (tyyppi=9),
      !  avulla
	!
	!
	!
C                          KAP1(i,j,k)=Ceff1(i,j,k)*Ro(apu1)*V
C                          KAP2(i,j,k)=KAP1(i,j,k)
	!
	!
	!
	!
				KAP1(i,j,k)=(Ceff1(i,j,k)*ParTih*TilOsuus*V)+
     &				        (KavCp*KavTih*(1-TilOsuus)*V)
                  KAP2(i,j,k)=KAP1(i,j,k)
                        else
      ! 
      ! 
      ! 
                          KAP1(i,j,k)=Cp(apu1)*Ro(apu1)*V
                          KAP2(i,j,k)=KAP1(i,j,k)
                        END IF
			END DO
		END DO
	END DO
	RETURN
	END SUBROUTINE
 	!__________________________________________________________________________
      !
	!Subprogram to calculate the values of the secondary MATA(730,730) and the present matrix
      !MATB(730,730) arvot
      !
	SUBROUTINE MATRIISIT(MATA,MATB,MENKERROI,LASVALI,KAP1,KAP2,G)
	IMPLICIT NONE
      INTEGER, PARAMETER :: tarkkuus = SELECTED_REAL_KIND(7,12)
	REAL (tarkkuus), INTENT(IN) :: MENKERROI,KAP1(9,9,9),KAP2(9,9,9),
     &                               G(1000,1000),LASVALI	                            
	REAL (tarkkuus), INTENT(OUT) :: MATA(730,730),MATB(730,730)
	INTEGER :: i,j,k,t,u,v,w,apux,apuy(6),apu1,apu2,p
	REAL (tarkkuus) :: Gkok	
      !
	!Putting zero the values of the matrix
	!
	DO i=1,730
		DO j=1,730
			MATA(i,j)=0
			MATB(i,j)=0
		END DO
	END DO	
	!
	! TO DEFINE THE VALUES OF THE KNOTS WHICH ARE CONNECTED TO " INSIDE POINTS "
	!
	! The coordinates of inside points are defined (1,1,0)
	! 
	! Apux is calculated from the coordinates of 
	! the knot and apuy from the neighbour coordinates.
      !
 	DO i=1,9
		DO j=1,9	
			apux=i*100+j*10+1
			apuy(1)=i*100+j*10+2
			apuy(2)=110
			apuy(3)=i*100+(j+1)*10+1
			apuy(4)=i*100+(j-1)*10+1
			apuy(5)=(i+1)*100+j*10+1
			apuy(6)=(i-1)*100+j*10+1
      !
      !
              DO p=1,6
                  IF (apuy(p).GT.1000) THEN
                      apuy(p)=100
                  END IF
              END DO 
      !
	! apu1
		    apu1=(j-1)*9+i
      !	
	! (=G)
      !
			Gkok=0
			DO t=1,6
                 IF ((G(apux,apuy(t)).GT.0).AND.(apuy(t).GE.110).AND.
     &                (apuy(t).LT.1000)) THEN
				Gkok=G(apux,apuy(t))+Gkok
                 END IF
			END DO !END DO t=1,6
      !
      ! Calculate the diagonal elements of the matrix MATA ja MATB 
      !
				MATA(apu1,apu1)=1+LASVALI*MENKERROI*Gkok/
     &							KAP1(i,j,1)
				MATB(apu1,apu1)=1-(LASVALI*(1-MENKERROI))*Gkok/
     &							KAP2(i,j,1)
      !
	! Calculate the elements non diagonal of the matrix 
      !  
      ! (1,1,1)=1 and (1,3,3)=25.
      !
			DO t=1,6
				u=apuy(t)/100
				v=(apuy(t)-u*100)/10
				w=(apuy(t)-u*100-v*10)
      !
	! Kun naapurina on sisilmapiste (1,1,0) niin normaalilaskentaa 
	! ei voida kytt, vaan lukuarvo apu2 on annettava eli...
      !
      ! apu2
				IF (t.EQ.2) THEN
					apu2=730
				ELSE
					apu2=(w-1)*81+(v-1)*9+u
				END IF
      !
      ! The initial matrix is defined if the conductance between the knot and the
	!  neighbour is non-zero.
      !				     				
				IF ((G(apux,apuy(t)).GT.0).AND.(apuy(t).GE.110).AND.
     &                (apuy(t).LT.1000)) THEN 
				MATA(apu1,apu2)= -LASVALI*MENKERROI*
     &							G(apux,apuy(t))/KAP1(i,j,1)
				MATB(apu1,apu2)=LASVALI*(1-MENKERROI)*
     &							G(apux,apuy(t))/KAP2(i,j,1)
				END IF
			END DO !End DO t=1,6
		END DO !End DO j=1,9
	END DO !End DO i=1,9
	!END Ilmasolmuun yhteydess olevat solmut
	!
	!
	! DEFINING THE KNOTS, WHO ARE NOT AT THE EDGE, TO THE MATRIX G (=CENTRAL KNOTS).
	! Naapurien laskennan kautta tulevien virheellisten koordinaattien
	! konduktanssit on nollattu jo aiemmassa aliohjelmassa (KONDUKTANSSI)
	!
	! Additional variables are defined to store the conductances at the middle of 
	! the knots to 2-dimensional table. Apux is calculated from the coordinates of 
	! the knot and apuy from the neighbour coordinates.
      !
	DO i=1,9
		DO j=1,9
			DO k=2,8
			apux=i*100+j*10+k
			apuy(1)=i*100+j*10+k-1
			apuy(2)=i*100+j*10+k+1
			apuy(3)=i*100+(j-1)*10+k
			apuy(4)=i*100+(j+1)*10+k
			apuy(5)=(i-1)*100+j*10+k
			apuy(6)=(i+1)*100+j*10+k
      !
      !
              DO p=1,6
                  IF (apuy(p).GT.1000) THEN
                      apuy(p)=100
                  END IF
              END DO 
      !
      ! apu1
			apu1=(k-1)*81+(j-1)*9+i
	!
	!  (=G)
      !
			Gkok=0
			DO t=1,6
                 IF ((G(apux,apuy(t)).GT.0).AND.(apuy(t).GE.111).AND.
     &                (apuy(t).LT.1000)) THEN
				Gkok=G(apux,apuy(t))+Gkok
                 END IF
			END DO
      !
      ! Calculate the diagonal elements of the matrix MATA and MATB 
      !
				MATA(apu1,apu1)=1+LASVALI*MENKERROI*Gkok/
     &                            KAP1(i,j,k)
				MATB(apu1,apu1)=1-(LASVALI*(1-MENKERROI))*Gkok/
     &							KAP2(i,j,k)
      !
      ! Caculate the elements non diagonal
			DO t=1,6
				u=apuy(t)/100
				v=(apuy(t)-u*100)/10
				w=(apuy(t)-u*100-v*10)
      !
      ! neighbour node apu2
				apu2=(w-1)*81+(v-1)*9+u
      !
      ! Muodostetaan matriisin alkio, jos solmupisteen
      ! ja naapurin vlille on mritelty nollasta poikkeava
      ! konduktanssi.
      !				     
				IF  ((G(apux,apuy(t)).GT.0).AND.(apuy(t).GE.111).AND.
     &                 (apuy(t).LT.1000)) THEN 
					MATA(apu1,apu2)= -LASVALI*MENKERROI*
     &								G(apux,apuy(t))/KAP1(i,j,k)
					MATB(apu1,apu2)=LASVALI*(1-MENKERROI)*
     &								G(apux,apuy(t))/KAP2(i,j,k)
				END IF
			  END DO !End DO t=1,6
			END DO !End DO k=2,8
		END DO !End DO j=1,9
	END DO !End DO i=1,9	 
	! END "KESKIPISTEET"
      !
      !
	! "TO DEFINE VALUES OF THE KNOTS IN THE MATRIX 
	! A AND B " THAT HAVE CONTACT TO " THE OUTSIDE  POINT".
	! "OUTSIDE POINT " = THE MOST INNER POINTS IN THE WALL (k=9).
      !
	! The outside point is defined point (1,0,0,0), which is not in use now,
	!
	! The wrong conductances calculated from the neighbour are set to zero before
	! at the subroutine (KONDUKTANSSI)
	!
	! Additional variables are defined to store the conductances at the middle 
	! of the knots to 2-dimensional table. Apux is calculated from the coordinates 
	! of the knot and apuy from the neighbour coordinates.
      !
	DO i=1,9
		DO j=1,9
			apux=i*100+j*10+9
			apuy(1)=i*100+j*10+8
			apuy(2)=1000
			apuy(3)=i*100+(j+1)*10+9
			apuy(4)=i*100+(j-1)*10+9
			apuy(5)=(i+1)*100+j*10+9
			apuy(6)=(i-1)*100+j*10+9
      !
      !Suodatetaan laittomia indeksien arvoja pois
              DO p=1,6
                  IF (apuy(p).GT.1000) THEN
                      apuy(p)=100
                  END IF
              END DO 
      !
      ! Solmupisteen jrjestysluku
			apu1=648+(j-1)*9+i

	! (=G)
			Gkok=0
			DO t=1,6
                IF ((G(apux,apuy(t)).GT.0).AND.(apuy(t).GE.111).AND.
     &                (apuy(t).LE.1000)) THEN
				 Gkok=G(apux,apuy(t))+Gkok
                END IF
			END DO
      !
      ! MATA and MATB 
      !
		    MATA(apu1,apu1)=1+LASVALI*MENKERROI*Gkok/KAP1(i,j,9)
			MATB(apu1,apu1)=1-(LASVALI*(1-MENKERROI))*Gkok/
     &					    KAP2(i,j,9)
      !
	! 
			DO t=1,6
	!  (1,0,0,0) 
	! 
				IF (t.EQ.2) THEN
				GO TO 160
				ELSE
      !
	! i:n ja j:n arvot saadaan suoraan loopista, mutta k:n arvo on 
	! mritettv kullekin t:n arvolle erikseen apuy(t):st eli... 
				u=apuy(t)/100
				v=(apuy(t)-u*100)/10
				w=(apuy(t)-u*100-v*10)
      !
      ! 
				apu2=(w-1)*81+(v-1)*9+u
      !
      ! 
      ! 
      ! 
      !	
				IF ((G(apux,apuy(t)).GT.0).AND.(apuy(t).GE.111).AND.
     &               (apuy(t).LT.1000)) THEN 
					MATA(apu1,apu2)= -LASVALI*MENKERROI*
     &								G(apux,apuy(t))/KAP1(i,j,9)
					MATB(apu1,apu2)=LASVALI*(1-MENKERROI)*
     &								G(apux,apuy(t))/KAP2(i,j,9)
					END IF
160				END IF
			END DO !End DO t=1,6
		END DO !End DO j=1,9
	END DO !End DO i=1,9
     	! END 'Sisimmiset' solmut
      !
	! Defining the inside point to the matrix A and B. Because the temperature at
	! inside is constant the multiplication factor of the matrix is 1.
      !
	MATA(730,730)=1
	!
	MATB(730,730)=1
      !
	RETURN
	END SUBROUTINE
	!
      !_______________________________________________________________________________________
      !
      !Subprogram to calculate the load of the matrix tehomatriisit 'MATEHO' and 'MBTEHO'
      !
  	SUBROUTINE TEHOMAT(P,LASVALI,MENKERROI,KAP1,KAP2,MATEHO,MBTEHO)
      IMPLICIT NONE
      INTEGER, PARAMETER :: tarkkuus = SELECTED_REAL_KIND(7,12)
      REAL (tarkkuus), INTENT(IN) :: P,MENKERROI,KAP1(9,9,9),
     &							   KAP2(9,9,9),LASVALI 	                    
      REAL (tarkkuus), INTENT(OUT) :: MATEHO(730,1),MBTEHO(730,1)
      INTEGER :: i,j,k,m,n
      !
      ! Putting the values of the tehomatriisit matrix equal zero
        DO m=1,730
           MATEHO(m,1)=0
           MBTEHO(m,1)=0
        END DO
      !
      ! Calculate tehomatriisit MAteho(730,1) and MBteho(730,1)
        n=0
        DO k=1,9
                DO j=1,9
                        DO i=1,9
                         n=n+1
      ! Tehomatriisien alkiot [1,81] ovat nollasta poikkeavia,
      ! koska lmpkuorma osuu vain seinlohkon pintaan (P=yhden solmun pintaan
      ! osuva lmpkuorma [W])
                         IF (n.LE.81) THEN
                         MATEHO(n,1)=-LASVALI*MENKERROI*P/
     &                                KAP1(i,j,k)
                         MBTEHO(n,1)=LASVALI*(1-MENKERROI)*P/
     &                               KAP2(i,j,k)
                         ELSE
      ! Tehomatriisien alkiot [82,730] ovat nollia
                         MATEHO(n,1)=0
                         MBTEHO(n,1)=0
                         END IF
                        END DO !END DO i=1,9
                END DO !END DO j=1,9
        END DO !END DO k=1,9
        MATEHO(730,1)=0
        MBTEHO(730,1)=0
        END SUBROUTINE

      !_______________________________________________________________________________________
	! subprogram to calculate the matrix MATB and T kertolaskun, eli MATB*T(n) = TULOS1
	!
	SUBROUTINE MATKER(MATB,T,TULOS1)
	IMPLICIT NONE
      INTEGER, PARAMETER :: tarkkuus = SELECTED_REAL_KIND(7,12)
	REAL (tarkkuus),INTENT(IN) :: MATB(730,730),T(730,1)
	REAL (tarkkuus),INTENT(OUT) :: TULOS1(730,1)
	INTEGER :: rivi,sarake
	DO rivi=1,730
	TULOS1(rivi,1)=0
		DO sarake=1,730
		TULOS1(rivi,1)=TULOS1(rivi,1)+MATB(rivi,sarake)*T(sarake,1)
		END DO
	END DO
	RETURN
	END SUBROUTINE
	!
	!_______________________________________________________________________________
      ! Subprogram to calculate the sum of the matrix M1 and M2 , eli
      !            M1 + M2 = TULOS2
      ! MATB*T(n) + MBteho = TULOS2
      !
      SUBROUTINE MATSUMMA(M1,M2,TULOS2)
      IMPLICIT NONE
      INTEGER, PARAMETER :: tarkkuus = SELECTED_REAL_KIND(7,12)
      REAL (tarkkuus),INTENT(IN) :: M1(730,1),M2(730,1)
      REAL (tarkkuus),INTENT(OUT) :: TULOS2(730,1)
      INTEGER :: rivi
      DO rivi=1,730
        TULOS2(rivi,1)=0
        TULOS2(rivi,1)=M1(rivi,1)+M2(rivi,1)
      END DO
      RETURN
      END SUBROUTINE
	!
	!________________________________________________________________________________
      ! Subprogram to calculate the gap M1 ja M2, eli 
      !                       M1 - M2 = TULOS3
      ! (MATB*T(n) + MBteho) - MAteho = TULOS3
      !
      SUBROUTINE MATERO(M1,M2,TULOS3)
      IMPLICIT NONE
      INTEGER, PARAMETER :: tarkkuus = SELECTED_REAL_KIND(7,12)
      REAL (tarkkuus),INTENT(IN) :: M1(730,1),M2(730,1)
      REAL (tarkkuus),INTENT(OUT) :: TULOS3(730,1)
      INTEGER :: rivi
      DO rivi=1,730
        TULOS3(rivi,1)=0
        TULOS3(rivi,1)=M1(rivi,1)-M2(rivi,1)
      END DO
      RETURN
      END SUBROUTINE
      !
	!________________________________________________________________________________
	! Subroutine calculated the temperatures T(n+1) of the time-step by solving X in 
	! the equation MATA*X=B 
      !
      !             MATA*X = B
      !        MATA*T(n+1) = MATB*T(n) + MBteho - MAteho
      !
      SUBROUTINE PARSE(B,N,X,MATA)
      IMPLICIT NONE
      INTEGER, PARAMETER :: tarkkuus = SELECTED_REAL_KIND(7,12)
      INTEGER,INTENT(IN) :: N
      REAL (tarkkuus), INTENT(IN) :: MATA(730,730),B(730)
      REAL (tarkkuus), INTENT(INOUT) :: X(730)
      REAL (tarkkuus) :: G(730),H(730),XI(730),XJ(730),EPS2,RP,BSQ,
     &                   RSQ,ANUM,ADEN,GG,DGG,GAM   
      INTEGER :: J,IRST,ITER
      INTEGER, PARAMETER :: NMAX = 730
      REAL, PARAMETER :: EPS = 1.E-7
      !
      EPS2=N*EPS**2
      IRST=0
1     IRST=IRST+1
      CALL ASUB(X,XI,MATA)
      RP=0.
      BSQ=0.
      DO 11 J=1,N
        BSQ=BSQ+B(J)**2
        XI(J)=XI(J)-B(J)
        RP=RP+XI(J)**2
11    CONTINUE
      CALL ATSUB(XI,G,MATA)
      DO 12 J=1,N
        G(J)=-G(J)
        H(J)=G(J)
12    CONTINUE
      DO 19 ITER=1,10*N
        CALL ASUB(H,XI,MATA)
        ANUM=0.
        ADEN=0.
        DO 13 J=1,N
          ANUM=ANUM+G(J)*H(J)
          ADEN=ADEN+XI(J)**2
13      CONTINUE
        IF(ADEN.EQ.0.)PAUSE 'very singular matrix'
        ANUM=ANUM/ADEN
        DO 14 J=1,N
          XI(J)=X(J)
          X(J)=X(J)+ANUM*H(J)
14      CONTINUE
        CALL ASUB(X,XJ,MATA)
        RSQ=0.
        DO 15 J=1,N
          XJ(J)=XJ(J)-B(J)
          RSQ=RSQ+XJ(J)**2
15      CONTINUE
        IF(RSQ.EQ.RP.OR.RSQ.LE.BSQ*EPS2)RETURN
        IF(RSQ.GT.RP)THEN
          DO 16 J=1,N
            X(J)=XI(J)
16        CONTINUE
          IF(IRST.GE.3)RETURN
          GO TO 1
        ENDIF
        RP=RSQ
        CALL ATSUB(XJ,XI,MATA)
        GG=0.
        DGG=0.
        DO 17 J=1,N
          GG=GG+G(J)**2
          DGG=DGG+(XI(J)+G(J))*XI(J)
17      CONTINUE
        IF(GG.EQ.0.)RETURN
        GAM=DGG/GG
        DO 18 J=1,N
          G(J)=-XI(J)
          H(J)=G(J)+GAM*H(J)
18      CONTINUE
19    CONTINUE
      PAUSE 'too many iterations'
      RETURN
      END
	!_______________________________________________________________________________________
      !
      ! Subroutine SPARSE uses subroutine ATSUB to do the multiplication of the 
	! transpose of matrix MATA(t) and vector X : MATA(t)*X = V
      !
      SUBROUTINE ASUB(X,V,MATA)
      IMPLICIT NONE
      INTEGER, PARAMETER :: tarkkuus = SELECTED_REAL_KIND(7,12)
	REAL (tarkkuus), INTENT(IN) :: MATA(730,730),X(730,1)
      REAL (tarkkuus), INTENT(OUT) :: V(730,1)
	INTEGER :: rivi,sarake
	DO rivi=1,730
	  V(rivi,1)=0
	  DO sarake=1,730
	   V(rivi,1)=V(rivi,1)+MATA(rivi,sarake)*X(sarake,1)
	  END DO
	END DO
	RETURN
	END SUBROUTINE
 	!_______________________________________________________________________________________
      !
      ! Subroutine SPARSE uses subroutine ATSUB to do the multiplication of the transpose of 
	! matrix MATA(t) and vector X : MATA(t)*X = V
      !    
      SUBROUTINE ATSUB(X,V,MATA)
      IMPLICIT NONE
      INTEGER, PARAMETER :: tarkkuus = SELECTED_REAL_KIND(7,12)
	REAL (tarkkuus), INTENT(IN) :: MATA(730,730),X(730,1)
      REAL (tarkkuus), INTENT(OUT) :: V(730,1)
	INTEGER :: rivi,sarake
	DO rivi=1,730
	  V(rivi,1)=0
	  DO sarake=1,730
	   V(rivi,1)=V(rivi,1)+MATA(sarake,rivi)*X(sarake,1)
          END DO
	END DO
	RETURN
	END SUBROUTINE
 	!_______________________________________________________________________________________
      !Aliohjelma ENERGIA laskee aika-askeleessa seinn sitoman 'Esit' tai luovuttaman 'Eluo' 
	!energian [kJ/m2]
	SUBROUTINE ENERGIA(VRK,TUNTI,LASVALI,HILMA,TnykK,ZoneT,Esit,
     &		   	       Eluo,Etot)
      IMPLICIT NONE
      INTEGER, PARAMETER :: tarkkuus = SELECTED_REAL_KIND(7,12)
	REAL (tarkkuus), INTENT(IN) ::VRK,TUNTI,LASVALI,HILMA,
     &							  TnykK,ZoneT
      REAL (tarkkuus), INTENT(OUT) ::Esit,Eluo,Etot 
	!
	IF (Zonet.GE.TnykK) THEN
	  Esit = 0.001*Hilma*(ZoneT - TnykK)*LASVALI
	  Eluo = 0
	ELSE
	  Esit = 0
	  Eluo = 0.001*Hilma*(ZoneT - TnykK)*LASVALI
	END IF
	Etot = Esit + Eluo
	RETURN
	END SUBROUTINE
	!_______________________________________________________________________________________
	!Subroutine calculated amount of the energy, which PCM-material can hold/give as a latent
	! heat in the region of melting/solidification [TtrSul-Tdiff,TtrSul+Tdiff]
      ! Subroutine sets the phase of the PCM-material as a solid by 'TILA(9,9,9)=0'
	SUBROUTINE ALKUVARASTO(Cp,Ro,V,Tdiff,LATENTTI,Ekok,ParTih,
     &					   TilOsuus)
      IMPLICIT NONE
      INTEGER, PARAMETER :: tarkkuus = SELECTED_REAL_KIND(7,12)
	REAL (tarkkuus), INTENT(IN) ::Cp(10),Ro(10),V,Tdiff,LATENTTI,
     &							  ParTih,TilOsuus
      REAL (tarkkuus), INTENT(OUT) ::Ekok
	INTEGER i,j,k
	!
	!In the region of melting or solidification of the phase transition [Ttr-Tdiff,Ttr+Tdiff] 
      !the maximum energy hold/given of the knot-point is Ekok.
	!sitoma tai luovuttama maksimienergia 'Ekok'.
C	Ekok = Ro(9)*V*LATENTTI
	Ekok = ParTih*TilOsuus*V*LATENTTI
	RETURN
	END SUBROUTINE
	!_______________________________________________________________________________________
	!
      ! END subprogram
